## Perennials win , w/o Disease
require(odesolve)

ayr.bydv = function(t,parms,ic) {
  with(parms,{
    Ps =1;  Pi = 2; ps = 3; pi = 4; as = 5; ai =6;
    
    res = NULL

    for(i in t) {
      ic[ps] = log(ic[ps])
      ic[Ps] = log(ic[Ps])
      ic[as] = log(ic[as])
      
      ## Calculate Growing Season Dynamics
      z = lsoda(ic,(i+gs),bydvL.ode,wiyp)

      ## Exp up things
      z[,1+Ps] = exp(z[,1+Ps])
      z[,1+ps] = exp(z[,1+ps])
      z[,1+as] = exp(z[,1+as])
      
      ## Get End of Growing Season Values
      e.gs = dim(z)
      e.gs = z[ e.gs[1], 2:e.gs[2] ]
      e.gs = pmax(0,e.gs) ## Avoid negatives
      
      ## Calculate Next Year's IC
      
      ##  Birth's from sick and healthy adult prnls
      ic[ps] = ( bp*( e.gs[Ps] + epsilon*e.gs[Pi] ) )
      ic[pi] = 0      # No infected juveniles
      
      ## Over non-growing season survival from adult & juveniles
      ## becme next year's adults.
      ic[Ps] = ( sigmaPs*( e.gs[Ps] + e.gs[ps] ) )
      ic[Pi] = sigmaPi*( e.gs[Pi] + e.gs[pi] )
      
      ##  Birth's from sick and healthy adult prnls
      ic[as] = ( ba*( e.gs[as] + gamma*e.gs[ai]) )
      ic[ai] = 0     # No infected annuals

      res = rbind(res,z)
      
  }
    res = data.frame(res)
    names(z) = c("time","Ps","Pi","ps","pi","as","ai")
    invisible(res)
  }
       )
}

       
plot.bydv = function(z) {
  plot(c(0,max(z[,1])),c(0,max(z[,2:7])),type="n",
       xlab="Time in Yrs",ylab="Abundance")

  for( i in 1:3) {
    lines(z[,1],z[,2*i],col=i)
    lines(z[,1],z[,2*i+1],col=i,lty="33")
  }
  
}

plot.fun = function(z,fun=sqrt,...) {
  m = max(z[,2:7])
  plot( c(0,max(z[,1])), c(0,fun(m) ), type="n",
       xlab="Time in Yrs", ylab="Abundance",
       axes=F,...)
  z = zapsmall(z)
  for( i in 1:3) {
    lines(z[,1],fun(z[,2*i]),col=i)
    lines(z[,1],fun(z[,2*i+1]),col=i,lty="33")
  }
  axis(1)
  y = max(1,floor(m/1000))
  y = c(0,10,50,100,500,1000)*y
  axis(2,at=fun(y),labels=y,las=1)
  box()
}

bydvL.ode = function(t, ic, parms){
  with(as.list(c(parms)),{

    Ps = exp(ic[1])
    Pi = ic[2]
    ps = exp(ic[3])
    pi = ic[4]
    as = exp(ic[5])
    ai = ic[6]
    
    ## Parms:
    ## BetaP
    ## mu   
        
    ## number of aphid Vectors
    V = (lambda*ai + Pi + pi) +v.int
    
    ## L-V Competition eqn1
    eta.p = (1+ alpha.pp*(ps+rho.p*pi) + alpha.pa*(as+rho.a*ai)
             + alpha.pP*(Ps+rho.p*Pi))
    ## L-V Competition eqn2 
    eta.a = (1+ alpha.aa*(as+rho.a*ai) + alpha.ap*(ps+rho.p*pi)
             + alpha.pP*(Ps+rho.p*Pi))

    ## This ode version tracks susceptibles on log space
    
    ## Adult Perennials
    dlPs = -BetaP*V 
    dPi = +BetaP*V*Ps

    ## Juvenile Perennials
    dlps = (-mu.p*eta.p - BetaP*V)
    dpi = -mu.p*eta.p*rho.p*pi + BetaP*V*ps
    
    ## Annuals
    dlas = (-mu.a*eta.a - BetaA*V)
    dai = -mu.a*eta.a*rho.a*ai + BetaA*V*as
        
    list(c(dlPs=dlPs, dPi=dPi, dlps=dlps,
           dpi=dpi, dlas=dlas, dai=dai))
  }
       )
}
  
#palette(c("darkgreen","green","violet"))

ayp = list(bp = 45.0, ba = 250.0,
  sigmaPs = 0.88, sigmaPi = 0.77, phi=1.0,
  epsilon = 0.5, gamma = 0.11, gs = 0:40/100)

wiyp = c(BetaP=0.02, BetaA=0.03, lambda=1.5,
  mu.a=1,mu.p=0.5,
  alpha.pp=1.5e-3, alpha.pa=4.8e-4, alpha.pP=1.4,
  alpha.aa=1.5e-4, alpha.ap =4.8e-5, alpha.aP=1.4,
  rho.p=0.50,rho.a=0.11)

#palette(c("darkgreen","green","violet"))

go=function(ic,invade.index,invade.abundance=1000){
  x = ayr.bydv(0:30,ayp,ic)

  ic = x[,1]==30.0
  ic = x[ic,2:7]
  ic[invade.index] = invade.abundance
  ic = as.vector(unlist(ic))
  
  y = ayr.bydv(31:100,ayp,ic)
  
  z = rbind(x,y)
  z = data.frame(z)
  names(z) = c("time","Ps","Pi","ps","pi","as","ai")
  m = (z$time) %% 1
  b = m > 0.199 ## < 1e-3
  m = m < (0.2 + 1e-3)
  m = b & m
  y = z[m,]
  ## y = z 
  y[,2:7] = 1*y[,2:7]

  plot(y$time,y$Ps,col="green4",lwd=2,
       xlab = "", ylab="",axes=F,
       log="y", ylim=c(1e-1,2e5), ## ylim=c(0,80),
       type="l")
  lines(y$time,y$Pi,col="green4",lwd=2,lty="33")
  
  lines(y$time,y$ps,col="green",lwd=2)
  
  lines(y$time,y$as,col="magenta",lwd=2)
  lines(y$time,y$ai,col="magenta",lwd=2,lty="33")
  
  lines(y$time,y$pi,col="green",lwd=2,lty="22")
  invisible(list(full=z,end=y))
}
